local rt = RoosterTools

--------------------------------------------------------------------------
-- Attach event handlers to the RoosterTools global.
--------------------------------------------------------------------------
RoosterToolsEvent = {}

function RoosterToolsEvent.GROUP_ROSTER_UPDATE()
    PartyWatch:UpdateDB()
end

function RoosterToolsEvent.GUILD_ROSTER_UPDATE()
    RoosterTools:GuildRosterTableUpdate()
end

function RoosterToolsEvent.GROUP_FORMED(e, category, partyGuid)
    RoosterToolsDB.partyGuid = partyGuid
end

function RoosterToolsEvent.GROUP_JOINED(e, category, partyGuid)
    RoosterToolsDB.partyGuid = partyGuid
end

function RoosterToolsEvent.GROUP_LEFT(e)
    if (not IsInGroup()) then
        RoosterToolsDB.partyGuid = nil
    end
end

function RoosterToolsEvent.ENCOUNTER_START(e, encounterID, encounterName, difficultyID, groupSize)
    local group = RoosterToolsDB.groups[RoosterToolsDB.partyGuid]
    if (not group) then
        return
    end

    table.insert(group.encounters, {
        instance = group.currentInstance,
        encounterID = encounterID,
        encounterName = encounterName,
        difficultyID = difficultyID,
        groupSize = groupSize,
        started = time()
    })
end

function RoosterToolsEvent.ENCOUNTER_END(e, encounterID, encounterName, difficultyID, groupSize, success)
    local group = RoosterToolsDB.groups[RoosterToolsDB.partyGuid]
    if (not group) then
        return
    end

    -- Find the fight that is currently ending... We need to scan due to older instances allowing multiple fights to
    -- occur in parallel
    local fight = nil
    for k, v in pairs(group.encounters) do
        if (v.encounterID == encounterID and not v.ended) then
            fight = v
        end
    end

    if (fight.encounterID ~= encounterID) then
        logDebug("Uh.... Fight state db out of sync.")
        return
    end

    fight.ended = time()

    if (success == 1) then
        fight.success = true
    else
        fight.success = false
    end
end

function RoosterToolsEvent.PLAYER_LEAVING_WORLD()
    RoosterToolsDB.groupsSerialized = RoosterTools:Serialize(RoosterToolsDB.groups)
end

-- Attach to all events.
function RoosterToolsEvent:OnInitialize()
    rt:RegisterEvent("GROUP_ROSTER_UPDATE", RoosterToolsEvent.GROUP_ROSTER_UPDATE)
    rt:RegisterEvent("GROUP_FORMED", RoosterToolsEvent.GROUP_FORMED)
    rt:RegisterEvent("GROUP_JOINED", RoosterToolsEvent.GROUP_JOINED)
    rt:RegisterEvent("GROUP_LEFT", RoosterToolsEvent.GROUP_LEFT)

    rt:RegisterEvent("ENCOUNTER_START", RoosterToolsEvent.ENCOUNTER_START)
    rt:RegisterEvent("ENCOUNTER_END", RoosterToolsEvent.ENCOUNTER_END)

    rt:RegisterEvent("GUILD_ROSTER_UPDATE", RoosterToolsEvent.GUILD_ROSTER_UPDATE)

    rt:RegisterEvent("PLAYER_LEAVING_WORLD", RoosterToolsEvent.PLAYER_LEAVING_WORLD)
end
