-- Docs for lib-st 
-- https://www.wowace.com/projects/lib-st/pages/set-data/standard-dataset-format
-- Main RoosterTools Frame
local RoosterToolsFrame = CreateFrame("Frame", "RoosterToolsHolderFrame", UIParent);
RoosterToolsFrame:SetScale(1.15)

-- Attach the main frame to the global RoosterTools object
RoosterTools.rootFrame = RoosterToolsFrame

-- Define colors to be used with frames.
local COLORS = {
    ["white"] = {
        ["r"] = 0.8,
        ["g"] = 0.8,
        ["b"] = 0.8,
        ["a"] = 1.0
    },
    ["black"] = {
        ["r"] = 0.0,
        ["g"] = 0.0,
        ["b"] = 0.0,
        ["a"] = 1.0
    },
    ["red"] = {
        ["r"] = 1.0,
        ["g"] = 0.8,
        ["b"] = 0.8,
        ["a"] = 1.0
    },
    ["green"] = {
        ["r"] = 0.8,
        ["g"] = 1.0,
        ["b"] = 0.8,
        ["a"] = 1.0
    },
    ["highlight"] = {
        ["r"] = 0.8,
        ["g"] = 0.8,
        ["b"] = 0.8,
        ["a"] = 0.5
    },
}

local function createButton(name, parent, height, width, text, point, offsetx, offsety, click)
    local button = CreateFrame("Button", name, parent)
    button:SetPoint(point, parent, offsetx, offsety)
    button:SetHeight(height)
    button:SetWidth(width)
    button:SetText(text)
    button:SetNormalFontObject("GameFontNormal")

    local ntex = button:CreateTexture()
    ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
    ntex:SetTexCoord(0, 0.625, 0, 0.6875)
    ntex:SetAllPoints()
    button:SetNormalTexture(ntex)

    local htex = button:CreateTexture()
    htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
    htex:SetTexCoord(0, 0.625, 0, 0.6875)
    htex:SetAllPoints()
    button:SetHighlightTexture(htex)

    local ptex = button:CreateTexture()
    ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
    ptex:SetTexCoord(0, 0.625, 0, 0.6875)
    ptex:SetAllPoints()
    button:SetPushedTexture(ptex)

    button:SetScript("OnClick", click)

    return button
end

local function createCheckbox(name, parent, text, tooltip, point, offsetx, offsety, click) 
    local checkbox = CreateFrame("CheckButton", name, parent, "ChatConfigCheckButtonTemplate");
    checkbox:SetPoint(point, parent, offsetx, offsety)
    _G[checkbox:GetName() .. 'Text']:SetText(text);
    checkbox.tooltip = tooltip;

    if (click) then
        checkbox:SetScript("OnClick", click) 
    end

    return checkbox
end

local function createTextBox(name, parent, point, relativeTo, relativePoint, offsetx, offsety, keyup)
    local textbox = CreateFrame("EditBox", name, parent)
    textbox:SetPoint(point, relativeTo, relativePoint, offsetx, offsety)
    textbox:SetAutoFocus(false)
    textbox:SetFontObject("ChatFontNormal")
    textbox:SetSize(150, 22)
    textbox:SetTextInsets(10, 10, 10, 10)
    Mixin(textbox, BackdropTemplateMixin)
    textbox.backdropInfo = BACKDROP_TEXT_PANEL_0_16
    textbox:OnBackdropLoaded()

    if (keyup) then
        textbox:SetScript("OnKeyUp", keyup)
    end

    return textbox
end

local function createLabel(name, parent, text, point, relativeTo, relativePoint, offsetx, offsety) 
    local label = parent:CreateFontString(name, "OVERLAY", "GameTooltipText")
    label:SetText(text)
    label:SetPoint(point, relativeTo, relativePoint, offsetx, offsety)
    return label
end

local function buildColumn(t)
    local t = t

    local template = {
        ["name"] = "#",
        ["width"] = 15,
        ["align"] = "RIGHT",
        ["color"] = COLORS.white,
        ["colorargs"] = nil,
        ["bgcolor"] = COLORS.black,
        ["DoCellUpdate"] = nil
    }

    for k, v in pairs(t) do
        template[k] = v
    end

    return template
end

local function createScrollFrame()
    local frame = RoosterToolsFrame
    frame.mainsOnly = true

    -- Give the frame a background
    Mixin(frame, BackdropTemplateMixin)
    frame.backdropInfo = BACKDROP_PARTY_32_32
    frame:OnBackdropLoaded()

    frame:SetMovable(true)
    frame:EnableMouse(true)
    frame:RegisterForDrag("LeftButton")
    frame:SetScript("OnDragStart", frame.StartMoving)
    frame:SetScript("OnDragStop", frame.StopMovingOrSizing)

    frame:SetWidth(670) -- Set these to whatever height/width is needed 
    frame:SetHeight(500) -- for your Texture
    frame:SetPoint("CENTER", 0, 0)
    frame:Hide()

    frame:SetScript("OnShow", function()
        C_GuildInfo.GuildRoster()
    end)

    local columns = {buildColumn({
        ["name"] = "#",
        ["width"] = 15
    }), buildColumn({
        ["name"] = "Rank",
        ["width"] = 100,
        ["align"] = "LEFT"
    }), buildColumn({
        ["name"] = "Main",
        ["width"] = 100,
        ["align"] = "LEFT",
        ["color"] = function(data, cols, realrow, column, table)
            local main = data[realrow][3]

            for k, v in pairs(RoosterToolsDB.aliases) do
                if (v.alias == main) then
                    main = k
                    break
                end
            end
            
            local name = data[realrow][4]
            local nameOnly = name:gmatch("[^-]+")()

            if (nameOnly == main) then
                return COLORS.green
            else
                return COLORS.red
            end
    
        end,
    }), buildColumn({
        ["name"] = "Name",
        ["width"] = 150,
        ["align"] = "LEFT",
    }), buildColumn({
        ["name"] = "Other",
        ["width"] = 50
    }), buildColumn({
        ["name"] = "Raids",
        ["width"] = 50
    }), buildColumn({
        ["name"] = "Absent",
        ["width"] = 50
    }), buildColumn({
        ["name"] = "Raid %",
        ["width"] = 50
    }), buildColumn({
        ["name"] = "Score",
        ["width"] = 50,
        ["color"] = function(data, cols, realrow, column, table)
            local score = data[realrow][9]

            if (score == 0) then
                return COLORS.red
            else
                return COLORS.green
            end
        end
    })}

    -- local tabardBackgroundUpper, tabardBackgroundLower, tabardEmblemUpper, tabardEmblemLower, tabardBorderUpper, tabardBorderLower = GetGuildTabardFileNames()

    local st = LibStub("ScrollingTable"):CreateST(columns, 25, 15, COLORS.highlight, frame)
    RoosterTools.guildRosterTable = st

    local characterFrame = CreateFrame("Frame", "RoosterToolsCharacterFrame", frame);
    -- Give the frame a background
    Mixin(characterFrame, BackdropTemplateMixin)
    characterFrame.backdropInfo = BACKDROP_DIALOG_32_32
    characterFrame:OnBackdropLoaded()

    characterFrame:SetWidth(300) -- Set these to whatever height/width is needed 
    characterFrame:SetHeight(500) -- for your Texture
    characterFrame:SetPoint("TOPLEFT", frame, "TOPRIGHT", 0, 0)
    characterFrame:Hide()

    local characterName = characterFrame:CreateFontString(characterFrame:GetName() .. "MainLabel", "OVERLAY",
                              "GameFontNormalLarge")
    characterName:SetText("")
    characterName:SetPoint("TOPLEFT", 20, -20)

    local aliasLabel = createLabel(characterFrame:GetName() .. "AliasLabel", characterFrame, "Alias", "TOPLEFT", characterName, "BOTTOMLEFT", 0, -10)
    local aliasText = createTextBox(characterFrame:GetName() .. "AliasText", characterFrame, "TOPLEFT", aliasLabel, "TOPRIGHT", 5, 0,
        function(parent, char) 
            local text = parent:GetText()
            if (text == "") then
                RoosterToolsDB.aliases[characterName:GetText()] = { alias = nil, updated = time() }
            else
                RoosterToolsDB.aliases[characterName:GetText()] = { alias = parent:GetText(), updated = time() }
            end

            RoosterTools:GuildRosterTableUpdate()
        end)

    local mainLabel = createLabel(characterFrame:GetName() .. "MainLabel", characterFrame, "Main", "TOPLEFT", aliasLabel, "BOTTOMLEFT", 20, -20)
    local mainText = createLabel(characterFrame:GetName() .. "MainLabel", characterFrame, "", "TOPLEFT", mainLabel, "TOPRIGHT", 5, 0)
    mainText:SetJustifyH("LEFT")

    st:RegisterEvents({
        ["OnClick"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
            if (not realrow) then
                return false
            end

            local name = data[realrow][4]
            name = name:gmatch("[^-]+")()

            characterFrame:Hide()

            local cName = name

            if (select(1, ...) == "RightButton") then
                cName = characterName:GetText()

                -- Add an alt
                if (cName == name) then
                    if (RoosterToolsDB.mains[cName]) then
                        RoosterToolsDB.mains[cName].name = nil
                        RoosterToolsDB.mains[cName].time = time()
                    end
                    mainLabel:SetText("")
                    mainText:SetText(name)
                else
                    RoosterToolsDB.mains[characterName:GetText()] = {name=name, time=time()}
                    mainLabel:SetText("Main")
                    mainText:SetText(name)
                end

                RoosterTools:GuildRosterTableUpdate()
            end

            characterName:SetText(cName)

            if (RoosterToolsDB.mains[cName] and RoosterToolsDB.mains[cName].name) then
                mainLabel:SetText("Main")
                mainText:SetText(RoosterToolsDB.mains[cName].name)
                aliasLabel:Hide()
                aliasText:Hide()
            else
                local alts = ""
                for k, v in pairs(RoosterToolsDB.mains) do
                    if (v.name == cName) then
                        alts = alts .. k .. "\n"
                    end
                end

                mainLabel:SetText("Alts")
                mainText:SetText(alts)
                aliasLabel:Show()

                local alias = RoosterToolsDB.aliases[characterName:GetText()]
                if (alias and alias.alias) then
                    aliasText:SetText(alias.alias)
                else
                    aliasText:SetText("")
                end
                aliasText:Show()
            end

            characterFrame:Show()
        end
    });

    local mainsOnlyCheckbox = createCheckbox("RoosterToolsHolderFrameMainsOnly", frame, "Mains", "Toggle mains only", "TOPRIGHT", -50, -15, function(frame, checkbox, state, a) 
        if (RoosterTools.rootFrame.mainsOnly) then
            RoosterTools.rootFrame.mainsOnly = false
        else
            RoosterTools.rootFrame.mainsOnly = true
        end

        RoosterTools:GuildRosterTableUpdate()
    end)

    mainsOnlyCheckbox:SetChecked(true)

    local guildMembersLabel = createLabel(characterFrame:GetName() .. "GuildMembers", frame, "Guild Mains", "TOPLEFT", frame, "BOTTOMLEFT", 15, 30)
    RoosterTools.rootFrame.guildMembers = createLabel(characterFrame:GetName() .. "GuildMembers2", frame, "-", "TOPLEFT", guildMembersLabel, "TOPRIGHT", 15, 0)

    createButton("RoosterToolsHolderFrameCloseButton", frame, 35, 175, "Close", "BOTTOMRIGHT", 0, 15,
        function(frame, button, down)
            RoosterToolsHolderFrame:Hide()
        end)
end

createScrollFrame()
